ReadWriteStream subclass: #Console
	instanceVariableNames: 'frame para transcript'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shrinking-Support'!

!Console methodsFor: 'initialization' stamp: 'shrink 8/27/2004 12:56'!
initInFrame: rect

	| box |
	
	Smalltalk at: #CurrentConsole put: self.
	
	frame := rect topLeft extent: (rect width @ (rect height/2)).
	frame _ frame insetBy: 2.  "Leave room for border"
	para _ Paragraph withText: self contents asText
				style: TextStyle default
				compositionRectangle: ((frame insetBy: 4) withHeight: 9999)
				clippingRectangle: frame
				foreColor: self black backColor: self white.
				
	box := (rect topLeft + (0 @ (rect height/2))) extent: (rect width @ (rect height/2)).
	transcript := TranscriptConsole newInFrame: box.
	
	Console cleanProcesses.

! !


!Console methodsFor: 'accessing' stamp: 'shrink 8/27/2004 08:13'!
clear
	Display fill: (frame insetBy: -2) fillColor: self black;
			fill: frame fillColor: self white.
	self on: (String new: 100); endEntry! !

!Console methodsFor: 'accessing' stamp: 'shrink 8/27/2004 15:40'!
endEntry
	| c d cb |
	c _ self contents.
	
	Display extent ~= DisplayScreen actualScreenSize ifTrue:
		["Handle case of user resizing physical window"
		DisplayScreen startUp.
		frame _ frame intersect: Display boundingBox.
		^ self clear; show: c].
	para setWithText: c asText
		style: TextStyle default
		compositionRectangle: ((frame insetBy: 4) withHeight: 9999)
		clippingRectangle: frame
		foreColor: self black backColor: self white.
	d _ para compositionRectangle bottom - frame bottom.
	d > 0 ifTrue:
		["Scroll up to keep all contents visible"
		cb _ para characterBlockAtPoint: para compositionRectangle topLeft
											+ (0@(d+para lineGrid)).
		self on: (c copyFrom: cb stringIndex to: c size).
		readLimit_ position_ collection size.
		^ self endEntry].
	para display! !

!Console methodsFor: 'accessing' stamp: 'shrink 8/27/2004 08:13'!
show: anObject
	self nextPutAll: anObject asString; endEntry! !


!Console methodsFor: 'command line' stamp: 'shrink 8/27/2004 08:13'!
confirm: queryString 
	| choice |
	[true]
		whileTrue: 
			[choice _ self request: queryString , '
Please type yes or no followed by return'.
			choice first asUppercase = $Y ifTrue: [^ true].
			choice first asUppercase = $N ifTrue: [^ false]]! !

!Console methodsFor: 'command line' stamp: 'shrink 8/27/2004 10:38'!
readEvalPrint
	| line |

	[#('quit' 'exit' 'done' ) includes: (line _ self request: '>')]
		whileFalse: [
			self cr; show: (
				[Compiler evaluate: line notifying: EvalError new logged: false]
					ifError:  [:err :ex | 
						| stack |
						stack := (thisContext stackOfSize: 30)
							inject: '' into: [:str :item | str, String cr, item asString ].
						err, String cr, stack 
					]
				)
		]! !

!Console methodsFor: 'command line' stamp: 'shrink 8/27/2004 08:37'!
request: prompt

	| startPos char contents | 
	self cr; show: prompt.
	startPos _ position.
	[
	[
		Processor yield.
		ProcessorScheduler relinquishProcessorForMicroseconds: 10000.
		Sensor keyboardPressed.
	] whileFalse.
	(char _ Sensor keyboard) = Character cr]
		whileFalse:
		[char = Character backspace
			ifTrue: [readLimit _ position _ (position - 1 max: startPos)]
			ifFalse: [
				Sensor commandKeyPressed 
					ifTrue: [ 
						(char = $v ) 
							ifTrue: [ self nextPutAll: Clipboard clipboardText ]
						]
					ifFalse: [ self nextPut: char ].				
				].
		self endEntry].
	contents _ self contents.
	^ contents copyFrom: startPos + 1 to: contents size! !


!Console methodsFor: 'private' stamp: 'shrink 8/27/2004 08:13'!
black
	Display depth = 1 ifTrue: [^ Bitmap with: 16rFFFFFFFF "Works without color support"].
	^ Color black! !

!Console methodsFor: 'private' stamp: 'shrink 8/27/2004 08:13'!
white
	Display depth = 1 ifTrue: [^ Bitmap with: 0 "Works without color support"].
	^ Color white! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Console class
	instanceVariableNames: ''!

!Console class methodsFor: 'instance creation' stamp: 'shrink 8/27/2004 08:15'!
newInFrame: frame

	| transcript |
	transcript _ self on: (String new: 100).
	transcript initInFrame: frame.
	^ transcript clear! !


!Console class methodsFor: 'utilities' stamp: 'shrink 8/27/2004 12:54'!
cleanProcesses 

	Transcript cr; show: 'Cleaning processes...'; cr.
	(Process allInstances select: [:p | 
		( {	
			EventSensor default interruptWatcherProcess.
			Processor activeProcess.
			WeakArray runningFinalizationProcess.
			Processor backgroundProcess
		} includes: p ) not ])
		do: [:p | p terminate ].
		
	(Process allInstances select: [:p | p isTerminated not ])
		do: [:p | Transcript show: p priority; show: ' - '; show: p; cr. ].		
		
	Transcript cr.
		
	! !

!Console class methodsFor: 'utilities' stamp: 'shrink 8/27/2004 14:09'!
open

	[ 
		Processor yield.
	(self newInFrame: Display boundingBox)
		show: 'Type ''exit'' to exit the console.';
		readEvalPrint.
	] fork.
	


		
	! !


Object subclass: #EvalError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shrinking-Support'!

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:35'!
first
! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 12:49'!
isKindOf: aClass

	aClass name = #SyntaxError 
		ifTrue: [ ^ true ].
	^ super isKindOf: aClass.! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:30'!
notify: aString at: location in: sourceStream

	Transcript 
		cr; show: 'Syntax error: '; show: aString; cr.
		! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:35'!
select

! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:34'!
selectFrom: first to: last

	^ Interval new.
		! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:34'!
selectionInterval

! !

!EvalError methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 10:37'!
text

	^ Text new! !


ReadWriteStream subclass: #TranscriptConsole
	instanceVariableNames: 'frame para'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Shrinking-Support'!

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 07:50'!
black
	Display depth = 1 ifTrue: [^ Bitmap with: 16rFFFFFFFF "Works without color support"].
	^ Color black! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 07:52'!
clear
	Display fill: (frame insetBy: -2) fillColor: self black;
			fill: frame fillColor: self white.
	self on: (String new: 100); endEntry! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 15:37'!
endEntry
	| c d cb |
	c _ self contents.
	
	Display extent ~= DisplayScreen actualScreenSize ifTrue:
		["Handle case of user resizing physical window"
		DisplayScreen startUp.
		frame _ frame intersect: Display boundingBox.
		^ self clear; show: c].
	para setWithText: c asText
		style: TextStyle default
		compositionRectangle: ((frame insetBy: 4) withHeight: 9999)
		clippingRectangle: frame
		foreColor: self black backColor: self white.
	d _ para compositionRectangle bottom - frame bottom.
	d > 0 ifTrue:
		["Scroll up to keep all contents visible"
		cb _ para characterBlockAtPoint: para compositionRectangle topLeft
											+ (0@(d+para lineGrid)).
		self on: (c copyFrom: cb stringIndex to: c size).
		readLimit_ position_ collection size.
		^ self endEntry].
	para display! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 08:04'!
initInFrame: rect

	frame _ rect insetBy: 2.  "Leave room for border"
	para _ Paragraph withText: self contents asText
				style: TextStyle default
				compositionRectangle: ((frame insetBy: 4) withHeight: 9999)
				clippingRectangle: frame
				foreColor: self black backColor: self white! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 08:14'!
release

	Transcript removeDependent: self.! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 08:12'!
update: event

	event = #clearText
		ifTrue: [ self clear ].
		
	event = #appendEntry
		ifTrue: [ 
			(StandardFileStream fileNamed: 'transcript') setToEnd; nextPutAll: Transcript contents; close.
			self nextPutAll: Transcript contents.
			self endEntry.
		]! !

!TranscriptConsole methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 07:52'!
white
	Display depth = 1 ifTrue: [^ Bitmap with: 0 "Works without color support"].
	^ Color white! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TranscriptConsole class
	instanceVariableNames: ''!

!TranscriptConsole class methodsFor: 'as yet unclassified' stamp: 'shrink 8/27/2004 14:03'!
newInFrame: frame

	| transcript |
	
	Processor yield.
	
	transcript _ self on: (String new: 100).
	transcript initInFrame: frame.
	
	Transcript addDependent: transcript.
	
	^ transcript clear! !
